from abc import ABC, abstractmethod
from typing import Any


class AbstractAPIClient(ABC):

    def __init__(self, api_key: str | None = None, api_version: str | None = None):
        self._check_required_packages()
        self.api_key = api_key
        self.api_version = api_version

    @property
    def base_api_url(self) -> str:
        return f"https://api.upassist.cloud/{self.api_version}/"

    @abstractmethod
    def _check_required_packages(self) -> None:
        return

    @abstractmethod
    def _request(
        self,
        method: str,
        url: str,
        params: dict | None = None,
        headers: dict | None = None,
        json: dict | None = None,
    ) -> Any:
        return

    def request(
        self,
        method: str,
        url: str,
        params: dict | None = None,
        headers: dict | None = None,
        json: dict | None = None,
    ) -> Any:
        if headers is None:
            headers = {}
        if self.api_key:
            headers["Authorization"] = f"Bearer {self.api_key}"
        return self._request(
            method=method,
            url=url,
            params=params,
            headers=headers,
            json=json,
        )



import functools
from typing import Type

from upassist import config
from upassist.client import AbstractAPIClient, SyncAPIClient


def attribute_required(attribute: str):
    def wrapper(func):
        @functools.wraps(func)
        def inner(self, *args, **kwargs):
            if not getattr(self, attribute):
                raise ValueError(f"`{attribute}` is required to perform object action")
            return func(self, *args, **kwargs)

        return inner

    return wrapper


heartbeat_slug_required = attribute_required("heartbeat_slug")


class Heartbeat:

    def __init__(
        self,
        heartbeat_slug: str | None = None,
        api_key: str | None = None,
        api_version: str | None = None,
        api_client_cls: Type[AbstractAPIClient] = SyncAPIClient,
    ):
        self.heartbeat_slug = heartbeat_slug
        self.api_client = api_client_cls(
            api_key=api_key or config.API_KEY,
            api_version=api_version or config.API_VERSION,
        )

    def list(
        self, q: str | None = None, page: int | None = None, per_page: int | None = None
    ):
        return self.api_client.request(
            "GET",
            "https://api.upassist.cloud/v1/heartbeats",
            params={
                "q": q,
                "page": page,
                "per_page": per_page,
            },
        )

    @heartbeat_slug_required
    def detail(self):
        return self.api_client.request(
            "GET", f"https://api.upassist.cloud/v1/heartbeats/{self.heartbeat_slug}"
        )

    @heartbeat_slug_required
    def pause(self):
        return self.api_client.request(
            "PATCH",
            f"https://api.upassist.cloud/v1/heartbeats/{self.heartbeat_slug}/pause",
        )

    @heartbeat_slug_required
    def unpause(self):
        return self.api_client.request(
            "PATCH",
            f"https://api.upassist.cloud/v1/heartbeats/{self.heartbeat_slug}/unpause",
        )

    @heartbeat_slug_required
    def delete(self):
        return self.api_client.request(
            "DELETE", f"https://api.upassist.cloud/v1/heartbeats/{self.heartbeat_slug}"
        )

    @heartbeat_slug_required
    def event(self):
        return self.api_client.request(
            "GET", f"https://heartbeats.upassist.cloud/api/event/{self.heartbeat_slug}"
        )



from . import config
from .client import AbstractAPIClient, AsyncAPIClient, SyncAPIClient
from .entities import Heartbeat

__all__ = (
    "Heartbeat",
    "config",
    "SyncAPIClient",
    "AbstractAPIClient",
    "AsyncAPIClient",
)
